#include <dos.h>

#ifndef __I2C_
#define __I2C_


#define Adr_Sx		0x30C	// Basic adress, access to all the registers defined in S1
#define Adr_S1		0x30D	// Adresse of the register S1;

#define Tempo		2000


int 	Control;

// Test the PIN bit

int I2C_PIN()
{
	int Val=inportb(Adr_S1);
	if ((Val & 0x80)==0) return (0);
	return (1);
}

// Test the LRB bit (Acknoledged ?)


int I2C_LRB()
{
	int Val=inportb(Adr_S1);
	if ((Val & 0x08)==0) return (0);
	return (1);
}

// Set the bit ESO (Enable Serial Output) to 0 or 1

void Set_ESO(int Val)
{
	if (Val==1) Control = Control | 0x40;
	else Control = Control & 0xBF;
	outportb (Adr_S1,Control);
}

void Set_ENI(int Val)
{
	if (Val==1) Control = Control | 0x08;
	else Control = Control & 0xF7;
	outportb (Adr_S1,Control);
}

// Test if the I2C line is busy (1 if busy, 0 else)

int I2C_Busy ()
{
	int val=inportb(Adr_S1);
	if ((val & 0x01)==0) return (1);
	else return (0);

}


// Initialize the PCF8584

void I2C_Init ()
{

	outportb (0x312,0x04);	// reset du PCF8584 Bit 2
	delay (1);
	outportb (0x312,0xFF);
	delay (1);
	outportb (Adr_S1,0x80);		// Access to register S0
	delay (1);
	outportb (Adr_Sx,0x55);		// Load 0x55 in register S0, Adress become 0xAA
	delay (1);
	outportb (Adr_S1,0xA0);		// Access to register S2
	delay (1);
	outportb (Adr_Sx,0x18);		// System clock = 8MHz SCL=90KHz
	delay (1);
	outportb (Adr_S1,0xC1);		// Enable serial output, ACK =1
	delay (1);
}



void Nnop (long int Num)
{
	long int i;
	for (i=0;i<Num;i++)
	{
		asm {nop;}

	}
}


// Send the byte Data at the adress Adr
// Return 0  if the transmission success
// Return -1 if the line I2C is busy (Transmission failed)
// Return -2 if the PIN bit never switch to 0
// return -3 if no aknoledgement is received from the slave (Adress is wrong ?)


int I2C_Send (int Adr,int Data)
{
	int TimeOut;					// Timeout

	outportb (Adr_S1,0xC1);			// Configure S1 register
	Nnop (Tempo);

	TimeOut=0;
	while (I2C_Busy()) {			// Wait while the bus is busy
		TimeOut++;
		if (TimeOut==200) {			// Ckeck TimeOut
			outportb (Adr_S1,0xC3);
			return (-1);
		}
	}

	outportb (Adr_Sx,Adr & 0xFE);			// Transfert the adress of the slave
	Nnop (Tempo);
	outportb (Adr_S1,0xC5);			// Generate the 'START' condition

	TimeOut=0;
	while (I2C_PIN()==1) {			// Wait for PIN bit
		TimeOut++;
		if (TimeOut==200) {			// Check TimeOut
			outportb (Adr_S1,0xC3);
			return (-2);
		}
	}

	if (I2C_LRB()!=0) {				// Slave acknoledged ?
			outportb (Adr_S1,0xC3);
			return (-3);
		}


	outportb (Adr_Sx,Data);			// Transfert the data to the slave

	TimeOut=0;
	while (I2C_PIN()==1) {			// Wait for PIN bit
		TimeOut++;
		if (TimeOut==200) {			// Check TimeOut
			outportb (Adr_S1,0xC3);
			return (-4);
		}
	}

	if (I2C_LRB()!=0) {				// Slave acknoledged ?
			outportb (Adr_S1,0xC3);
			return (-5);
		}

	outportb (Adr_S1,0xC3);			// Generates 'STOP' condition
	Nnop (Tempo);
	return (0);
}



// receive the byte Data from the adress Adr
// Return Data  if the transmission success
// Return -1 if the line I2C is busy (Transmission failed)
// Return -2 if the PIN bit never switch to 0
// return -3 if no aknoledgement is received from the slave (Adress is wrong ?)


int I2C_Receive (char Adr)
{
	int TimeOut;					// Timeout
	int Data;

	/*** Add this line for the PCF8574 ***/

//	I2C_Send (Adr,0xFF),			// Configure the port in input

	outportb (Adr_S1,0xC1);			// Configure S1 register
	Nnop (Tempo);

	outportb (Adr_Sx,Adr | 0x01);	// Load slave adress into S0 register
	Nnop (Tempo);

	TimeOut=0;
	while (I2C_Busy()) {			// Wait while the bus is busy
		TimeOut++;
		if (TimeOut==200) {			// Ckeck TimeOut
			outportb (Adr_S1,0xC3);
			return (-1);
		}
	}

	outportb (Adr_S1,0xC5);			// Generate the 'START' condition
	Nnop (Tempo);

	TimeOut=0;
	while (I2C_PIN()==1) {			// Wait for PIN bit
		TimeOut++;
		if (TimeOut==200) {			// Check TimeOut
			outportb (Adr_S1,0xC3);
			return (-2);
		}
	}

	if (I2C_LRB()!=0) {				// Slave acknoledged ?
			outportb (Adr_S1,0xC3);
			return (-3);
		}

	outportb (Adr_S1,0x40);			// Set ACK bit to 0
	Nnop (Tempo);

	Data=inportb (Adr_Sx);			// Read the Dummy data from S0 register
									// Ignore this value

	TimeOut=0;
	while (I2C_PIN()==1) {			// Wait for PIN bit
		TimeOut++;
		if (TimeOut==200) {			// Check TimeOut
			outportb (Adr_S1,0xC3);
			return (-2);
		}
	}


	Data=inportb (Adr_Sx);			// Read data byte from S0 register
	Nnop (Tempo);
	outportb (Adr_S1,0xC3);			// Generates 'STOP' condition
	Nnop (Tempo);
	return (Data);
}



#endif

