#ifndef __CLAVIER_H__
#define __CLAVIER_H__

#include "I2C.H"

// prototypes des fonctions :
int AdaptationLC(int);
int LectureI2C();
int LectureClavierI2C(int);

//////////////////////////////////////
//
// Fonction AdaptationLC(int arg)
//
// Paramte d'entre :
// 		int arg : nligne/colone I2C  adapter
//
// Valeur retourne :
//		int retour : n rel de la ligne/colone
//
//
// Description :
// 		Renvoie la ligne ou la colone de la touche appuye
// 		verifie que 2  touches n'as t apuyes
//////////////////////////////////////
int AdaptationLC (int arg)
{
  int i=0, numero=0, nbz=0;
  int retour;

  do
  {
	if( (arg & 1) == 0)  //si on a un 0 sur le bit test
	{
		numero = i;	     // sauvegarder le numero du bit qui est  0
		nbz++;           // compte le nombre de 0
	}
	i++;                 //incremente le numero du bit test
	arg>>=1;             //decale l'argument pour test le bit suivant
  } while(i<4);          //tant qu'on a pas un bit  0

  if(nbz==1)             //si on a un seule bit  0
  {
	retour = 3 - numero; //b3b2b1b0 -> b0b1b2b3
  }
  else retour = -1;      //indice d'erreur s'est pass

  return retour;
}

//////////////////////////////////////
//
// Fonction int LectureI2C()
//
// Valeur retourne :
//		int retour : valeur de la touche appuyee
//					 (-1 si erreur)
//
//
// Description : Lit le port I2C et dtecte
//		quelle touche du clavier  t appuye
//
//////////////////////////////////////
int LectureI2C()
{
	int colone, ligne, OnTouche=-1;
	int code[4][4]={1 ,2,3 ,15,    //ligne 0 du clavier
					4 ,5,6 ,14,    //ligne 1 du clavier
					7 ,8,9 ,13,    //ligne 2 du clavier
					10,0,11,12};   //ligne 3 du clavier


	// balayage des colones du clavier
	I2C_Send(0x44,0xF0);
	colone = I2C_Receive(0x45);	// lecture de la colone
	//traitement sur la colone uniquement
	colone = colone>>4;         // $xxxx+yyyy -> colone = $xxxx
	// trouver le n de colone
	colone = AdaptationLC(colone);

	// balayage des ligne du clavier
	I2C_Send(0x44, 0x0F);
	ligne = I2C_Receive(0x45);	// lecture de la ligne
	//traitement sur la ligne uniquement (pas besoin de dcaller)
	// trouver le n de ligne
	ligne = AdaptationLC(ligne);

	// si on a une ligne et une colone de trouve
	// (donc une et une seule touche)
	if( ligne!=-1 && colone!=-1)
		OnTouche = code[ligne][colone];	// rcupration du n de la touche

	return OnTouche;
}

//////////////////////////////////////
//
// Fonction int LectureClavierI2C(int bloquant, int nbCycle=100)
//
// Valeur d'entre :
//		int nbCycle : En mode non-bloquant, nombre de lecture
//					  sur l'I2C  effectuer avant de passer
//					   travers la fonction pour continuer le
//					  programme.
//					  '0' pour le mode bloquant (attente de l'appuis
//					  d'une touche).
//
// Valeur retourne :
//		int retour : valeur de la touche appuyee
//					 (-1 si aucune touche, si erreur, si multi-touche)
//
//
// Description : Gere la lecture de la valeur
//		envoyee sur l'I2C par le clavier
//
//////////////////////////////////////
int LectureClavierI2C (int nbCycle)
{
	int touche=-1, lecture, flag=0, Cycle = 0;

	// mode bloquant
	if (!nbCycle)	// tant qu'une touche ne sera pas appuye, on attend
	{
		// Tant qu'on a pas 15 fois (flag = 15)
		// la meme touche appuye en continue
		while (flag != 15 && touche == -1)
		{
			lecture = LectureI2C();	// rcupration du n de touche
			if (LectureI2C() == lecture) // si la lecture = la prcendante
				flag ++; 	// incrmente le flag SI meme touche depuis la lecture prcdante
			else flag = 0;	// sinon on rinitialise le flag

			if (lecture == -1) // si pas de touche appuye
				flag = 0; 	   // rinitialisation du flag
		}

		touche = lecture; // on mmorise la touche valide
	}
	else	// mode non-bloquant
			// On attend qu'un certains temps avant de passer
			//  l'instruction suivante du programme appelant

	// nbCycle correspond au timeout
	while (Cycle < nbCycle+15 && touche == -1)
	{
		lecture = LectureI2C();	// rcupration du n de la touche
		if (LectureI2C() == lecture) // si la lecture = la prcdante
			flag++; // incrmentation du flag
		else flag = 0; // sinon on rinitialise le flag

		if (flag == 15 && lecture != -1) // si on a la mme touche appuye 15 cycle conscutifs
			touche = lecture;	// on mmorise la touche valide

		if (lecture == -1) // si pas de touche appuye
				flag = 0;  // rinitialisation du flag

		if (Cycle > nbCycle) // dans les 15 derniers cylcles, il n'y a plus
			Cycle = nbCycle+14;  // le temps suffisant pour dtecter une touche
								// on force donc la sortie de la fonction
		Cycle ++;	// incrmentation du nombre de cycle
	}

	return touche;
}


#endif	//!__CLAVIER_H__