

#include "I2C.H"
#ifndef __LCD__
#define __LCD__


#define ADR_I2C_LCD 0x00

char PositionLigne;
char ClrLCD();
char EcrireLCD(char *, char, char);

////////////////////////////////////////
//
// Fonction : ClrLCD()
//
// Description : Efface l'ecran LCD
//
// Valeur retournee : 0 si transmission reussie
//
////////////////////////////////////////
char ClrLCD()
{
 int retour=0;
 int i;
 for (i = 0 ; i < 80 ; i++)
	 retour = I2C_Send (ADR_I2C_LCD,0x20);	// ecriture du caractere 'ESPACE' partout
								// qui efface l'ecran

 PositionLigne = 1;		// indicateur position ligne 1

 return retour;
}

////////////////////////////////////////
//
// Fonction : 	EcrireLCD(		char *,
//						char ArgNbCaractere,
//						char ArgPositionLigne = 0)
//
// Description : Ecrire sur l'ecran LCD
//
// Valeur retournee : 0 si transmission reussie
//
// Variables d'entree :
//			char * ArgChaine : chaine de caractere a ecrire
//			char ArgNbCaractere : Nombre de caractere dans chaine
//			char ArgPositionLigne : indication sur quelle ligne ecrire
//			char ArgPositionColone : indication sur quelle colone ecrire
//
//	Remarques : Rendu au bout d'un ligne, on va automatiquement
//				a la ligne suivante puis on reboucle sur la ligne 1
//				si on arrive a la derniere ligne
//
////////////////////////////////////////
char EcrireLCD(char *ArgChaine,char ArgNbCaractere,char ArgPositionLigne)
{
 char i=0, retour=0, posColone=1;

 // si une ligne a ete definnie (1 a 4)
 if (ArgPositionLigne > 0 && ArgPositionLigne <= 4)
 {
	retour = I2C_Send (ADR_I2C_LCD,ArgPositionLigne);	// on va a cette ligne
	PositionLigne = ArgPositionLigne;
 }

 while (i<ArgNbCaractere && retour==0)
 {
	if (posColone == 20)	// si on arrive en bout de ligne
	{
		// On incremente le numero de ligne de 1 a 4 puis on reboucle a 1
		if (PositionLigne < 4)
			PositionLigne ++;
		else PositionLigne = 1;

		// on deplace le curseur du LCD vers la nouvelle ligne
		retour = I2C_Send (ADR_I2C_LCD,PositionLigne);
	}
	retour=I2C_Send(80,ArgChaine[i]);
	i++;
 }
 PositionLigne = 1;

 return retour;
}

#endif